/*****************************************************************************/
/*
  data.c: save data from simulation
*/
/*****************************************************************************/
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mrdplot/mrdplot.h"

#include "useful/useful.h"
#include "dynamics.h"

#include "data.h"

/*****************************************************************************/

/*****************************************************************************/

int n_points = 0;
float data[MAX_N_POINTS*N_CHANNELS];

char *names[N_CHANNELS];
char *units[N_CHANNELS];

/*****************************************************************************/

void init_data( Dynamics *s )
{
  int i;

  /*
  printf( "N_CHANNELS: %d; M_L_TORSO_BOTTOM: %d; last: %d\n", N_CHANNELS,
	  M_L_TORSO_BOTTOM, M_R_ANKLE_COMMAND );
  */

  for( i = 0; i < N_CHANNELS; i++ )
    {
      names[i] = "xxx";
      units[i] = "xxx";
    }

  names[M_TIME] = "time";

  names[M_X_XR] = "x_xr";
  names[M_X_AB] = "x_ab";
  names[M_X_AKL] = "x_akl";
  names[M_X_AKR] = "x_akr";
  names[M_X_AH] = "x_ah";

  names[M_X_XRD] = "x_xrd";
  names[M_X_ABD] = "x_abd";
  names[M_X_AKLD] = "x_akld";
  names[M_X_AKRD] = "x_akrd";
  names[M_X_AHD] = "x_ahd";

  names[M_U_AAL] = "u_aal";
  names[M_U_AKL] = "u_akl";
  names[M_U_AHL] = "u_ahl";
  names[M_U_AAR] = "u_aar";
  names[M_U_AKR] = "u_akr";
  names[M_U_AHR] = "u_ahr";

  names[M_FBSLX] = "FBSLx";
  names[M_FBSLY] = "FBSLy";
  names[M_FBSRX] = "FBSRx";
  names[M_FBSRY] = "FBSRy";

  names[M_XR] = "xr";
  names[M_YR] = "yr";
  names[M_AR] = "ar";
  names[M_XC] = "xc";
  names[M_YC] = "yc";
  names[M_XB] = "xb";
  names[M_YB] = "yb";
  names[M_AB] = "ab";
  names[M_XAL] = "xal";
  names[M_YAL] = "yal";
  names[M_XSL] = "xsl";
  names[M_YSL] = "ysl";
  names[M_ASL] = "asl";
  names[M_XKL] = "xkl";
  names[M_YKL] = "ykl";
  names[M_XTHL] = "xthl";
  names[M_YTHL] = "ythl"; 
  names[M_ATHL] = "athl"; 
  names[M_XAR] = "xar";
  names[M_YAR] = "yar";
  names[M_XSR] = "xsr";
  names[M_YSR] = "ysr";
  names[M_ASR] = "asr";
  names[M_XKR] = "xkr";
  names[M_YKR] = "ykr";
  names[M_XTHR] = "xthr";
  names[M_YTHR] = "ythr"; 
  names[M_ATHR] = "athr"; 
  names[M_XH] = "xh";
  names[M_YH] = "yh";
  names[M_XTO] = "xto";
  names[M_YTO] = "yto";
  names[M_ATO] = "ato";
  names[M_XHEAD] = "xhead";
  names[M_YHEAD] = "yhead";

  names[M_XRD] = "xrd";
  names[M_YRD] = "yrd";
  names[M_XCD] = "xcd";
  names[M_YCD] = "ycd";
  names[M_XBD] = "xbd";
  names[M_YBD] = "ybd";
  names[M_XALD] = "xald";
  names[M_YALD] = "yald";
  names[M_XSLD] = "xsld";
  names[M_YSLD] = "ysld";
  names[M_XKLD] = "xkld";
  names[M_YKLD] = "ykld";
  names[M_XTHLD] = "xthld";
  names[M_YTHLD] = "ythld";
  names[M_XARD] = "xard";
  names[M_YARD] = "yard";
  names[M_XSRD] = "xsrd";
  names[M_YSRD] = "ysrd";
  names[M_XKRD] = "xkrd";
  names[M_YKRD] = "ykrd";
  names[M_XTHRD] = "xthrd";
  names[M_YTHRD] = "ythrd";
  names[M_XHD] = "xhd";
  names[M_YHD] = "yhd";
  names[M_XTOD] = "xtod";
  names[M_YTOD] = "ytod";
  names[M_XHEADD] = "xheadd";
  names[M_YHEADD] = "yheadd";

  names[M_WR] = "wr";
  names[M_WB] = "wb";
  names[M_WSL] = "wsl";
  names[M_WTHL] = "wthl";
  names[M_WSR] = "wsr";
  names[M_WTHR] = "wthr";
  names[M_WTO] = "wto";

  names[M_XCOM] = "xcom";
  names[M_YCOM] = "ycom";
  names[M_DX] = "dx";

  names[M_XCOMD] = "xcomd";
  names[M_YCOMD] = "ycomd";
  names[M_DXD] = "dxd";

  names[M_ENERGY] = "energy";

  names[M_RKE] = "rke";
  names[M_RPE] = "rpe";
  names[M_BKE] = "bke";
  names[M_BPE] = "bpe";
  names[M_SLKE] = "slke";
  names[M_SLPE] = "slpe";
  names[M_THLKE] = "thlke";
  names[M_THLPE] = "thlpe";
  names[M_SRKE] = "srke";
  names[M_SRPE] = "srpe";
  names[M_THRKE] = "thrke";
  names[M_THRPE] = "thrpe";
  names[M_TOKE] = "toke";
  names[M_TOPE] = "tope";
  names[M_KE] = "ke";
  names[M_PE] = "pe";

  names[M_FGRX] = "FGRx";
  names[M_FGRY] = "FGRy";
  names[M_FRBX] = "FRBx";
  names[M_FRBY] = "FRBy";

  names[M_XX_XR] = "xx_xr";
  names[M_XX_YR] = "xx_yr";
  names[M_XX_AR] = "xx_ar";
  names[M_XX_XB] = "xx_xb";
  names[M_XX_YB] = "xx_yb";
  names[M_XX_AB] = "xx_ab";
  names[M_XX_XSL] = "xx_xsl";
  names[M_XX_YSL] = "xx_ysl";
  names[M_XX_ASL] = "xx_asl";
  names[M_XX_XTHL] = "xx_xthl";
  names[M_XX_YTHL] = "xx_ythl"; 
  names[M_XX_ATHL] = "xx_athl"; 
  names[M_XX_XSR] = "xx_xsr";
  names[M_XX_YSR] = "xx_ysr";
  names[M_XX_ASR] = "xx_asr";
  names[M_XX_XTHR] = "xx_xthr";
  names[M_XX_YTHR] = "xx_ythr"; 
  names[M_XX_ATHR] = "xx_athr"; 
  names[M_XX_XTO] = "xx_xto";
  names[M_XX_YTO] = "xx_yto";
  names[M_XX_ATO] = "xx_ato";

  names[M_XX_XRD] = "xx_xrd";
  names[M_XX_YRD] = "xx_yrd";
  names[M_XX_ARD] = "xx_ard";
  names[M_XX_XBD] = "xx_xbd";
  names[M_XX_YBD] = "xx_ybd";
  names[M_XX_ABD] = "xx_abd";
  names[M_XX_XSLD] = "xx_xsld";
  names[M_XX_YSLD] = "xx_ysld";
  names[M_XX_ASLD] = "xx_asld";
  names[M_XX_XTHLD] = "xx_xthld";
  names[M_XX_YTHLD] = "xx_ythld";
  names[M_XX_ATHLD] = "xx_athld";
  names[M_XX_XSRD] = "xx_xsrd";
  names[M_XX_YSRD] = "xx_ysrd";
  names[M_XX_ASRD] = "xx_asrd";
  names[M_XX_XTHRD] = "xx_xthrd";
  names[M_XX_YTHRD] = "xx_ythrd";
  names[M_XX_ATHRD] = "xx_athrd";
  names[M_XX_XTOD] = "xx_xtod";
  names[M_XX_YTOD] = "xx_ytod";
  names[M_XX_ATOD] = "xx_atod";

  names[M_XX_XRDD] = "xx_xrdd";
  names[M_XX_YRDD] = "xx_yrdd";
  names[M_XX_ARDD] = "xx_ardd";
  names[M_XX_XBDD] = "xx_xbdd";
  names[M_XX_YBDD] = "xx_ybdd";
  names[M_XX_ABDD] = "xx_abdd";
  names[M_XX_XSLDD] = "xx_xsldd";
  names[M_XX_YSLDD] = "xx_ysldd";
  names[M_XX_ASLDD] = "xx_asldd";
  names[M_XX_XTHLDD] = "xx_xthldd";
  names[M_XX_YTHLDD] = "xx_ythldd";
  names[M_XX_ATHLDD] = "xx_athldd";
  names[M_XX_XSRDD] = "xx_xsrdd";
  names[M_XX_YSRDD] = "xx_ysrdd";
  names[M_XX_ASRDD] = "xx_asrdd";
  names[M_XX_XTHRDD] = "xx_xthrdd";
  names[M_XX_YTHRDD] = "xx_ythrdd";
  names[M_XX_ATHRDD] = "xx_athrdd";
  names[M_XX_XTODD] = "xx_xtodd";
  names[M_XX_YTODD] = "xx_ytodd";
  names[M_XX_ATODD] = "xx_atodd";

  names[M_NE_FSLTHLX] = "ne_FSLTHLx";
  names[M_NE_FSLTHLY] = "ne_FSLTHLy";
  names[M_NE_FTHLTOX] = "ne_FTHLTOx";
  names[M_NE_FTHLTOY] = "ne_FTHLTOy";
  names[M_NE_FSRTHRX] = "ne_FSRTHRx";
  names[M_NE_FSRTHRY] = "ne_FSRTHRy";
  names[M_NE_FTHRTOX] = "ne_FTHRTOx";
  names[M_NE_FTHRTOY] = "ne_FTHRTOy";

  units[M_TIME] = "s";

  units[M_X_XR] = "m";
  units[M_X_AB] = "r";
  units[M_X_AKL] = "r";
  units[M_X_AKR] = "r";
  units[M_X_AH] = "r";

  units[M_X_XRD] = "m/s";
  units[M_X_ABD] = "r/s";
  units[M_X_AKLD] = "r/s";
  units[M_X_AKRD] = "r/s";
  units[M_X_AHD] = "r/s";

  units[M_U_AAL] = "N/m";
  units[M_U_AKL] = "N/m";
  units[M_U_AHL] = "N/m";
  units[M_U_AAR] = "N/m";
  units[M_U_AKR] = "N/m";
  units[M_U_AHR] = "N/m";

  units[M_XC] = "m";
  units[M_YC] = "m";
  units[M_XB] = "m";
  units[M_YB] = "m";
  units[M_XSL] = "m";
  units[M_YSL] = "m";
  units[M_XKL] = "m";
  units[M_YKL] = "m";
  units[M_XTHL] = "m";
  units[M_YTHL] = "m";
  units[M_XH] = "m";
  units[M_YH] = "m";
  units[M_XTO] = "m";
  units[M_YTO] = "m";
  units[M_XHEAD] = "m";
  units[M_YHEAD] = "m";

  units[M_XCD] = "m/s";
  units[M_YCD] = "m/s";
  units[M_XBD] = "m/s";
  units[M_YBD] = "m/s";
  units[M_XSLD] = "m/s";
  units[M_YSLD] = "m/s";
  units[M_XKLD] = "m/s";
  units[M_YKLD] = "m/s";
  units[M_XTHLD] = "m/s";
  units[M_YTHLD] = "m/s";
  units[M_XHD] = "m/s";
  units[M_YHD] = "m/s";
  units[M_XTOD] = "m/s";
  units[M_YTOD] = "m/s";
  units[M_XHEADD] = "m/s";
  units[M_YHEADD] = "m/s";

  units[M_WR] = "r/s";
  units[M_WB] = "r/s";
  units[M_WSL] = "r/s";
  units[M_WTHL] = "r/s";
  units[M_WTO] = "r/s";

  units[M_XCOM] = "m";
  units[M_YCOM] = "m";
  units[M_DX] = "m";

  units[M_XCOMD] = "m/s";
  units[M_YCOMD] = "m/s";
  units[M_DXD] = "m/s";

  units[M_ENERGY] = "-";

  units[M_RKE] = "-";
  units[M_RPE] = "-";
  units[M_BKE] = "-";
  units[M_BPE] = "-";
  units[M_SLKE] = "-";
  units[M_SLPE] = "-";
  units[M_THLKE] = "-";
  units[M_THLPE] = "-";
  units[M_TOKE] = "-";
  units[M_TOPE] = "-";
  units[M_KE] = "-";
  units[M_PE] = "-";

  units[M_XX_XB] = "m";
  units[M_XX_YB] = "m";
  units[M_XX_XSL] = "m";
  units[M_XX_YSL] = "m";
  units[M_XX_XTHL] = "m";
  units[M_XX_YTHL] = "m";
  units[M_XX_XTO] = "m";
  units[M_XX_YTO] = "m";

  units[M_XX_XBD] = "m/s";
  units[M_XX_YBD] = "m/s";
  units[M_XX_XSLD] = "m/s";
  units[M_XX_YSLD] = "m/s";
  units[M_XX_XTHLD] = "m/s";
  units[M_XX_YTHLD] = "m/s";
  units[M_XX_XTOD] = "m/s";
  units[M_XX_YTOD] = "m/s";

  /*
  printf( "%d channels, %d MAX_N_POINTS\n", N_CHANNELS, MAX_N_POINTS );

  for( i = 0; i < N_CHANNELS; i++ )
    {
      printf( "%d %s %s\n", i, names[i], units[i] );
    }
  */
}

/*****************************************************************************/

int save_data( Dynamics *s )
{
  static int myindex = 0;
  
  if ( n_points >= MAX_N_POINTS )
    return 0;

  data[myindex + M_TIME] = (float) (s->time);

  data[myindex + M_X_XR] = (float) (s->state[X_XR]);
  data[myindex + M_X_AB] = (float) (s->state[X_AB]);
  data[myindex + M_X_AKL] = (float) (s->state[X_AKL]);
  data[myindex + M_X_AKR] = (float) (s->state[X_AKR]);
  data[myindex + M_X_AH] = (float) (s->state[X_AH]);

  data[myindex + M_X_XRD] = (float) (s->state[X_XRD]);
  data[myindex + M_X_ABD] = (float) (s->state[X_ABD]);
  data[myindex + M_X_AKLD] = (float) (s->state[X_AKLD]);
  data[myindex + M_X_AKRD] = (float) (s->state[X_AKRD]);
  data[myindex + M_X_AHD] = (float) (s->state[X_AHD]);

  data[myindex + M_U_AAL] = (float) (s->action[U_AAL]);
  data[myindex + M_U_AKL] = (float) (s->action[U_AKL]);
  data[myindex + M_U_AHL] = (float) (s->action[U_AHL]);
  data[myindex + M_U_AAR] = (float) (s->action[U_AAR]);
  data[myindex + M_U_AKR] = (float) (s->action[U_AKR]);
  data[myindex + M_U_AHR] = (float) (s->action[U_AHR]);

  data[myindex + M_FBSLX] = (float) (s->nr_b[XX_FBSLX+1]);
  data[myindex + M_FBSLY] = (float) (s->nr_b[XX_FBSLY+1]);
  data[myindex + M_FBSRX] = (float) (s->nr_b[XX_FBSRX+1]);
  data[myindex + M_FBSRY] = (float) (s->nr_b[XX_FBSRY+1]);

  data[myindex + M_XR] = (float) (s->xr);
  data[myindex + M_YR] = (float) (s->yr);
  data[myindex + M_AR] = (float) (s->ar);
  data[myindex + M_XC] = (float) (s->xc);
  data[myindex + M_YC] = (float) (s->yc);
  data[myindex + M_XB] = (float) (s->xb);
  data[myindex + M_YB] = (float) (s->yb);
  data[myindex + M_AB] = (float) (s->ab);
  data[myindex + M_XAL] = (float) (s->xal);
  data[myindex + M_YAL] = (float) (s->yal);
  data[myindex + M_XSL] = (float) (s->xsl);
  data[myindex + M_YSL] = (float) (s->ysl);
  data[myindex + M_ASL] = (float) (s->asl);
  data[myindex + M_XKL] = (float) (s->xkl);
  data[myindex + M_YKL] = (float) (s->ykl);
  data[myindex + M_XTHL] = (float) (s->xthl);
  data[myindex + M_YTHL] = (float) (s->ythl);
  data[myindex + M_ATHL] = (float) (s->athl);
  data[myindex + M_XAR] = (float) (s->xar);
  data[myindex + M_YAR] = (float) (s->yar);
  data[myindex + M_XSR] = (float) (s->xsr);
  data[myindex + M_YSR] = (float) (s->ysr);
  data[myindex + M_ASR] = (float) (s->asr);
  data[myindex + M_XKR] = (float) (s->xkr);
  data[myindex + M_YKR] = (float) (s->ykr);
  data[myindex + M_XTHR] = (float) (s->xthr);
  data[myindex + M_YTHR] = (float) (s->ythr);
  data[myindex + M_ATHR] = (float) (s->athr);
  data[myindex + M_XH] = (float) (s->xh);
  data[myindex + M_YH] = (float) (s->yh);
  data[myindex + M_XTO] = (float) (s->xto);
  data[myindex + M_YTO] = (float) (s->yto);
  data[myindex + M_ATO] = (float) (s->ato);
  data[myindex + M_XHEAD] = (float) (s->xhead);
  data[myindex + M_YHEAD] = (float) (s->yhead);

  data[myindex + M_XRD] = (float) (s->xrd);
  data[myindex + M_YRD] = (float) (s->yrd);
  data[myindex + M_XCD] = (float) (s->xcd);
  data[myindex + M_YCD] = (float) (s->ycd);
  data[myindex + M_XBD] = (float) (s->xbd);
  data[myindex + M_YBD] = (float) (s->ybd);
  data[myindex + M_XALD] = (float) (s->xald);
  data[myindex + M_YALD] = (float) (s->yald);
  data[myindex + M_XSLD] = (float) (s->xsld);
  data[myindex + M_YSLD] = (float) (s->ysld);
  data[myindex + M_XKLD] = (float) (s->xkld);
  data[myindex + M_YKLD] = (float) (s->ykld);
  data[myindex + M_XTHLD] = (float) (s->xthld);
  data[myindex + M_YTHLD] = (float) (s->ythld);
  data[myindex + M_XARD] = (float) (s->xard);
  data[myindex + M_YARD] = (float) (s->yard);
  data[myindex + M_XSRD] = (float) (s->xsrd);
  data[myindex + M_YSRD] = (float) (s->ysrd);
  data[myindex + M_XKRD] = (float) (s->xkrd);
  data[myindex + M_YKRD] = (float) (s->ykrd);
  data[myindex + M_XTHRD] = (float) (s->xthrd);
  data[myindex + M_YTHRD] = (float) (s->ythrd);
  data[myindex + M_XHD] = (float) (s->xhd);
  data[myindex + M_YHD] = (float) (s->yhd);
  data[myindex + M_XTOD] = (float) (s->xtod);
  data[myindex + M_YTOD] = (float) (s->ytod);
  data[myindex + M_XHEADD] = (float) (s->xheadd);
  data[myindex + M_YHEADD] = (float) (s->yheadd);

  data[myindex + M_WR] = (float) (s->wr);
  data[myindex + M_WB] = (float) (s->wb);
  data[myindex + M_WSL] = (float) (s->wsl);
  data[myindex + M_WTHL] = (float) (s->wthl);
  data[myindex + M_WSR] = (float) (s->wsr);
  data[myindex + M_WTHR] = (float) (s->wthr);
  data[myindex + M_WTO] = (float) (s->wto);

  data[myindex + M_XCOM] = (float) (s->xcom);
  data[myindex + M_YCOM] = (float) (s->ycom);
  data[myindex + M_DX] = (float) (s->dx);

  data[myindex + M_XCOMD] = (float) (s->xcomd);
  data[myindex + M_YCOMD] = (float) (s->ycomd);
  data[myindex + M_DXD] = (float) (s->dxd);

  data[myindex + M_ENERGY] = (float) (s->energy);

  data[myindex + M_RKE] = (float) (s->rke);
  data[myindex + M_RPE] = (float) (s->rpe);
  data[myindex + M_BKE] = (float) (s->bke);
  data[myindex + M_BPE] = (float) (s->bpe);
  data[myindex + M_SLKE] = (float) (s->slke);
  data[myindex + M_SLPE] = (float) (s->slpe);
  data[myindex + M_THLKE] = (float) (s->thlke);
  data[myindex + M_THLPE] = (float) (s->thlpe);
  data[myindex + M_SRKE] = (float) (s->srke);
  data[myindex + M_SRPE] = (float) (s->srpe);
  data[myindex + M_THRKE] = (float) (s->thrke);
  data[myindex + M_THRPE] = (float) (s->thrpe);
  data[myindex + M_TOKE] = (float) (s->toke);
  data[myindex + M_TOPE] = (float) (s->tope);
  data[myindex + M_KE] = (float) (s->ke);
  data[myindex + M_PE] = (float) (s->pe);

  data[myindex + M_FGRX] = (float) (s->nr_b[XX_FGRX+1]);
  data[myindex + M_FGRY] = (float) (s->nr_b[XX_FGRY+1]);
  data[myindex + M_FRBX] = (float) (s->nr_b[XX_FRBX+1]);
  data[myindex + M_FRBY] = (float) (s->nr_b[XX_FRBY+1]);

  data[myindex + M_XX_XR] = (float) (s->full_state[XX_XR]);
  data[myindex + M_XX_YR] = (float) (s->full_state[XX_YR]);
  data[myindex + M_XX_AR] = (float) (s->full_state[XX_AR]);
  data[myindex + M_XX_XB] = (float) (s->full_state[XX_XB]);
  data[myindex + M_XX_YB] = (float) (s->full_state[XX_YB]);
  data[myindex + M_XX_AB] = (float) (s->full_state[XX_AB]);
  data[myindex + M_XX_XSL] = (float) (s->full_state[XX_XSL]);
  data[myindex + M_XX_YSL] = (float) (s->full_state[XX_YSL]);
  data[myindex + M_XX_ASL] = (float) (s->full_state[XX_ASL]);
  data[myindex + M_XX_XTHL] = (float) (s->full_state[XX_XTHL]);
  data[myindex + M_XX_YTHL] = (float) (s->full_state[XX_YTHL]);
  data[myindex + M_XX_ATHL] = (float) (s->full_state[XX_ATHL]);
  data[myindex + M_XX_XSR] = (float) (s->full_state[XX_XSR]);
  data[myindex + M_XX_YSR] = (float) (s->full_state[XX_YSR]);
  data[myindex + M_XX_ASR] = (float) (s->full_state[XX_ASR]);
  data[myindex + M_XX_XTHR] = (float) (s->full_state[XX_XTHR]);
  data[myindex + M_XX_YTHR] = (float) (s->full_state[XX_YTHR]);
  data[myindex + M_XX_ATHR] = (float) (s->full_state[XX_ATHR]);
  data[myindex + M_XX_XTO] = (float) (s->full_state[XX_XTO]);
  data[myindex + M_XX_YTO] = (float) (s->full_state[XX_YTO]);
  data[myindex + M_XX_ATO] = (float) (s->full_state[XX_ATO]);

  data[myindex + M_XX_XRD] = (float) (s->full_state[XX_XR + N_XX_ACC]);
  data[myindex + M_XX_YRD] = (float) (s->full_state[XX_YR + N_XX_ACC]);
  data[myindex + M_XX_ARD] = (float) (s->full_state[XX_AR + N_XX_ACC]);
  data[myindex + M_XX_XBD] = (float) (s->full_state[XX_XB + N_XX_ACC]);
  data[myindex + M_XX_YBD] = (float) (s->full_state[XX_YB + N_XX_ACC]);
  data[myindex + M_XX_ABD] = (float) (s->full_state[XX_AB + N_XX_ACC]);
  data[myindex + M_XX_XSLD] = (float) (s->full_state[XX_XSL + N_XX_ACC]);
  data[myindex + M_XX_YSLD] = (float) (s->full_state[XX_YSL + N_XX_ACC]);
  data[myindex + M_XX_ASLD] = (float) (s->full_state[XX_ASL + N_XX_ACC]);
  data[myindex + M_XX_XTHLD] = (float) (s->full_state[XX_XTHL + N_XX_ACC]);
  data[myindex + M_XX_YTHLD] = (float) (s->full_state[XX_YTHL + N_XX_ACC]);
  data[myindex + M_XX_ATHLD] = (float) (s->full_state[XX_ATHL + N_XX_ACC]);
  data[myindex + M_XX_XSRD] = (float) (s->full_state[XX_XSR + N_XX_ACC]);
  data[myindex + M_XX_YSRD] = (float) (s->full_state[XX_YSR + N_XX_ACC]);
  data[myindex + M_XX_ASRD] = (float) (s->full_state[XX_ASR + N_XX_ACC]);
  data[myindex + M_XX_XTHRD] = (float) (s->full_state[XX_XTHR + N_XX_ACC]);
  data[myindex + M_XX_YTHRD] = (float) (s->full_state[XX_YTHR + N_XX_ACC]);
  data[myindex + M_XX_ATHRD] = (float) (s->full_state[XX_ATHR + N_XX_ACC]);
  data[myindex + M_XX_XTOD] = (float) (s->full_state[XX_XTO + N_XX_ACC]);
  data[myindex + M_XX_YTOD] = (float) (s->full_state[XX_YTO + N_XX_ACC]);
  data[myindex + M_XX_ATOD] = (float) (s->full_state[XX_ATO + N_XX_ACC]);

  data[myindex + M_XX_XRDD] = (float) (s->nr_b[XX_XR+1]); 
  data[myindex + M_XX_YRDD] = (float) (s->nr_b[XX_YR+1]); 
  data[myindex + M_XX_ARDD] = (float) (s->nr_b[XX_AR+1]); 
  data[myindex + M_XX_XBDD] = (float) (s->nr_b[XX_XB+1]); 
  data[myindex + M_XX_YBDD] = (float) (s->nr_b[XX_YB+1]); 
  data[myindex + M_XX_ABDD] = (float) (s->nr_b[XX_AB+1]); 
  data[myindex + M_XX_XSLDD] = (float) (s->nr_b[XX_XSL+1]); 
  data[myindex + M_XX_YSLDD] = (float) (s->nr_b[XX_YSL+1]); 
  data[myindex + M_XX_ASLDD] = (float) (s->nr_b[XX_ASL+1]); 
  data[myindex + M_XX_XTHLDD] = (float) (s->nr_b[XX_XTHL+1]); 
  data[myindex + M_XX_YTHLDD] = (float) (s->nr_b[XX_YTHL+1]); 
  data[myindex + M_XX_ATHLDD] = (float) (s->nr_b[XX_ATHL+1]); 
  data[myindex + M_XX_XSRDD] = (float) (s->nr_b[XX_XSR+1]); 
  data[myindex + M_XX_YSRDD] = (float) (s->nr_b[XX_YSR+1]); 
  data[myindex + M_XX_ASRDD] = (float) (s->nr_b[XX_ASR+1]); 
  data[myindex + M_XX_XTHRDD] = (float) (s->nr_b[XX_XTHR+1]); 
  data[myindex + M_XX_YTHRDD] = (float) (s->nr_b[XX_YTHR+1]); 
  data[myindex + M_XX_ATHRDD] = (float) (s->nr_b[XX_ATHR+1]); 
  data[myindex + M_XX_XTODD] = (float) (s->nr_b[XX_XTO+1]); 
  data[myindex + M_XX_YTODD] = (float) (s->nr_b[XX_YTO+1]); 
  data[myindex + M_XX_ATODD] = (float) (s->nr_b[XX_ATO+1]); 

  data[myindex + M_NE_FSLTHLX] = (float) (s->nr_b[XX_FSLTHLX+1]);
  data[myindex + M_NE_FSLTHLY] = (float) (s->nr_b[XX_FSLTHLY+1]);
  data[myindex + M_NE_FTHLTOX] = (float) (s->nr_b[XX_FTHLTOX+1]);
  data[myindex + M_NE_FTHLTOY] = (float) (s->nr_b[XX_FTHLTOY+1]);
  data[myindex + M_NE_FSRTHRX] = (float) (s->nr_b[XX_FSRTHRX+1]);
  data[myindex + M_NE_FSRTHRY] = (float) (s->nr_b[XX_FSRTHRY+1]);
  data[myindex + M_NE_FTHRTOX] = (float) (s->nr_b[XX_FTHRTOX+1]);
  data[myindex + M_NE_FTHRTOY] = (float) (s->nr_b[XX_FTHRTOY+1]);

  myindex += N_CHANNELS;
  n_points++;

  return 0;
}

/*****************************************************************************/

int write_matlab_file( char *filename )
{
  FILE *stream;
  int i;
  static int myindex = 0;

  stream = fopen( filename, "w" );
  if ( stream == NULL )
    {
      fprintf( stderr, "Can't open %s for write.\n", filename );
      exit( -1 );
    }
  for ( i = 0; i < n_points; i++ )
    {
      fprintf( stream, "%g ", data[myindex + M_TIME] );
      // add other stuff you want here
      fprintf( stream, "\n" );
      myindex += N_CHANNELS;
    }
  fclose( stream );

  return 0;
}

/*****************************************************************************/

int write_the_mrdplot_file( Dynamics *s )
{
  MRDPLOT_DATA *d; 

  d = malloc_mrdplot_data( 0, 0 );
  d->filename = generate_file_name();
  d->n_channels = N_CHANNELS;
  d->n_points = n_points;
  d->total_n_numbers = d->n_channels*d->n_points;
  d->frequency = (float) (1/s->time_step);
  d->data = data;
  d->names = names;
  d->units = units;

  write_mrdplot_file( d );

  // write_matlab_file( "dd" );

  return 0;
}

/*****************************************************************************/
