/************************************************************************/
/*
Example of linearizing dynamics.
*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include "useful/useful.h"
#include "dynamics.h"

/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/

#define KNEE_GOAL 0.2
#define KNEE_TORQUE 25.61048429046979

/************************************************************************/
/* main */

int main(int argc, char **argv)
{
  int i, j;
  double state[N_X];
  double save_state[N_X];
  double next_state_0[N_X];
  double next_state_1[N_X];
  double next_state_2[N_X];
  double action[N_U];
  double save_action[N_U];
  double delta = 1e-4; // often need to play with this value to see where
                      // you get a linearization that is insensitive to it.
  double a[N_X][N_X];
  double b[N_X][N_U];
  Dynamics *d;
	
  init_dynamics();
  d = create_dynamics();

  for( i = 0; i < N_X; i++ )
    {
      state[i] = d->desired_state[i];
    }

  // bend knees
  state[X_AKL] += 0.2;
  state[X_AKR] += 0.2;

  for( i = 0; i < N_X; i++ )
    {
      save_state[i] = state[i];
    }

  for( i = 0; i < N_U; i++ )
    {
      action[i] = 0;
    }

  // balance gravity
  action[U_AKL] = KNEE_TORQUE;
  action[U_AKR] = KNEE_TORQUE;

  for( i = 0; i < N_U; i++ )
    {
      save_action[i] = action[i];
    }

  set_state( d, 0.0, state );
  if ( 0 )
    {
      printf( "state: " );
      for( j = 0; j < N_X; j++ )
	{
          printf( "%20.15g ", d->state[j] );
	}
      printf( "\n" );
      printf( "action: " );
      for( j = 0; j < N_U; j++ )
	{
          printf( "%20.15g ", action[j] );
	}
      printf( "\n" );
    }
  integrate_one_step( d, action, next_state_0 );

  for( i = 0; i < N_X; i++ )
    {
      state[i] += delta;
      set_state( d, 0.0, state );
      if ( 0 )
	{
	  printf( "state: " );
	  for( j = 0; j < N_X; j++ )
	    {
	      printf( "%20.15g ", d->state[j] );
	    }
	  printf( "\n" );
	  printf( "action: " );
	  for( j = 0; j < N_U; j++ )
	    {
	      printf( "%20.15g ", action[j] );
	    }
	  printf( "\n" );
	}
      integrate_one_step( d, action, next_state_1 );
      state[i] = save_state[i] - delta;
      set_state( d, 0.0, state );
      if ( 0 )
	{
	  printf( "state: " );
	  for( j = 0; j < N_X; j++ )
	    {
	      printf( "%20.15g ", d->state[j] );
	    }
	  printf( "\n" );
	  printf( "action: " );
	  for( j = 0; j < N_U; j++ )
	    {
	      printf( "%20.15g ", action[j] );
	    }
	  printf( "\n" );
	}
      integrate_one_step( d, action, next_state_2 );
      for( j = 0; j < N_X; j++ )
	{
	  a[j][i] = (next_state_1[j] - next_state_2[j])/(2*delta);
	  if ( 0 )
	    printf( "%d %d %20.15g %20.15g %20.15g\n", j, i, a[j][i],
		    next_state_1[j], next_state_2[j] );
	}
      state[i] = save_state[i];
    }

  for( i = 0; i < N_U; i++ )
    {
      action[i] += delta;
      set_state( d, 0.0, state );
      integrate_one_step( d, action, next_state_1 );
      action[i] = save_action[i] - delta;
      set_state( d, 0.0, state );
      integrate_one_step( d, action, next_state_2 );
      for( j = 0; j < N_X; j++ )
	{
	  b[j][i] = (next_state_1[j] - next_state_2[j])/(2*delta);
	}
      action[i] = save_action[i];
    }

  printf( "A:\n" );
  for( i = 0; i < N_X; i++ )
    {
      for( j = 0; j < N_X; j++ )
	{
	  printf( "%20.15g ", a[i][j] );
	}
      printf( "\n" );
    }

  printf( "B:\n" );
  for( i = 0; i < N_X; i++ )
    {
      for( j = 0; j < N_U; j++ )
	{
	  printf( "%20.15g ", b[i][j] );
	}
      printf( "\n" );
    }

  return 0;
}

/************************************************************************/

